#!/bin/ksh
# 
# Run :     control_quality_buoy.ksh.v1 [Period] [Buoy_name] 
# Example:  control_quality_buoy.ksh.v1 022013 BOIA_SC    
# 
data=$1        # Data: YYYY1MM1_YYYY2MM2
station=$2     # Buoy name : cabofrio2,itajai,fortaleza, etc.

filein=/scratchin/grupos/oceano/home/rosio.camayo/validation/buoys/control_quality_buoy_goosbrasil/d-output/${station}_${data}_wave_wind.dat
fileout1=/scratchin/grupos/oceano/home/rosio.camayo/validation/buoys/control_quality_buoy_goosbrasil/d-output/${station}_${data}_hs_A
fileout2=/scratchin/grupos/oceano/home/rosio.camayo/validation/buoys/control_quality_buoy_goosbrasil/d-output/${station}_${data}_hs_B_spikes

#awk '{print substr($2,1,4), substr($2,6,2), substr($2,9,2), substr($3,1,2), substr($3,4,2), substr($3,7,2), $7, $9, $10}' $filein > temp


# PART 1: Eliminate lines containing NaN values and obvious errors of wave height (higher than 15.0 m & lower than 5 cm) -------

awk '$10!=-9999 && $10>0.05 && $10<15.0 {print $0}' $filein > temp

# --------------------------------------------------------

nlines=$(wc -l temp |awk '{print $1}') 

#  2. Eliminate spikes higher than 4*deviation standard of neighbours in interval of -+3 hours

cat << EOF > cutspikes.f90

!Program to eliminate outliers using deviation with neighbors data of ~ 3-+hours around (only using hours data, not considers minutes)

parameter (nlines=${nlines})
integer, dimension  (nlines) :: year,month,day,hour,minute
real, dimension  (nlines)    :: lat,lon,windsp,winddir,hs,pk,wavedir
real                         :: meanhsneighbor
integer                      :: flag

open(10, file='temp',form='formatted',status='unknown') 

do nl=1,nlines

! If date format is modified, please change this line:
read(10,*) lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),windsp(nl),winddir(nl),hs(nl),pk(nl),wavedir(nl)

enddo

close(10)

do nl=1,nlines


! PART 2: Eliminate spikes higher than 4*deviation standard of neighbours in interval of -+4 hours

hsorig=hs(nl)
sumhsneighbor=0.0
sumhsneighbor2=0.0
nn=0
  do nl_b=-30,30 ! Considering 30 neighbors around  

     nlcomp=nl+nl_b 

     if (nlcomp.lt.1.or.nlcomp.gt.nlines) nlcomp=99999
  
     if(nlcomp.ne.99999.and.nlcomp.ne.nl) then

!        write(55,*) "nlorig",nl,"vizinho",nl_b,"linhavizinho",nlcomp

        do hour_int=-4,4 ! Considering interval -+ 4 hours

             houri=hour(nl)+hour_int
             dayi=day(nl)

             if(houri.gt.23) then
                dayi=dayi+1
                houri=houri-24
             endif
             if(houri.lt.0) then
                dayi=dayi-1
                houri=houri+24
             endif

!                write(55,*) "hour_int",hour_int,"houri",houri,"dayi",dayi   
             if(day(nlcomp).eq.dayi.and.hour(nlcomp).eq.houri) then
                if(hs(nlcomp).le.10.) then
                  hsneighbor=hs(nlcomp) 
                  sumhsneighbor=hs(nlcomp)+sumhsneighbor
                  sumhsneighbor2=hs(nlcomp)**2+sumhsneighbor2
                  nn=nn+1
!                 write(55,*) "daycomp",day(nlcomp),"dayi",dayi,"hourcomp",hour(nlcomp),"houri",houri,"hscomp",hs(nlcomp)
                endif
             endif 

        end do

     endif 
  end do

  if (nn.eq.0) then ! Case 1: No neighbor
 
   meanhsneighbor=hsorig 
   devhsneighbor=0.0
   flag=1
  
  endif

  if (nn.eq.1) then ! Case 2: One neighbor

   meanhsneighbor=sumhsneighbor/float(nn)
   devhsneighbor=0.0
   flag=2

  endif

  if (nn.gt.1) then ! Case 3: More one neighbor

   meanhsneighbor=sumhsneighbor/float(nn)
   devhsneighbor=sqrt(sumhsneighbor2/float(nn)-(sumhsneighbor/float(nn))**2)
   flag=3
  
  endif

   if (flag.eq.1) then 
      if (hs(nl).le.8.0) then
         write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,F5.2)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),hs(nl)
      else
         write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,I4)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),-999
      endif 
   endif


   if (flag.eq.2) then
     if(abs(hs(nl)-meanhsneighbor).le.6.0) then
       write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,F5.2)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),hs(nl)
     else
     write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,I4)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),-999
     endif
   endif
        
   if (flag.eq.3) then
      if(abs(hs(nl)-meanhsneighbor).le.4*devhsneighbor) then
      write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,F5.2)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),hs(nl)
      else 
        if(abs(hs(nl)-meanhsneighbor).le.3.0) then
           write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,F5.2)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),hs(nl)
        else
           write(40,'(I5,1x,(2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,I4)') nl,lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),-999
        endif 
      endif
   endif

      write(20,'((2(F12.3,2x)),I4,1x,(4(I2,1x)),F5.2,1x,F12.4,1x,F12.4,1x,F12.4)') lat(nl),lon(nl),year(nl),month(nl),day(nl),hour(nl),minute(nl),hs(nl),meanhsneighbor,abs(hs(nl)-meanhsneighbor),4*devhsneighbor

! -------

end do


close(20)
close(40)
close(55)

stop
end

EOF


   pgf90 cutspikes.f90 -o cutspikes

    ./cutspikes
    
    /bin/rm -f cutspikes cutspikes.f90

    /bin/rm -f temp
    
    mv fort.20 $fileout1
    mv fort.40 $fileout2



